/*
 * Ricart.java
 *
 * Created on 20. November 2000, 19:42
 */
 


/** 
 *
 * @author  fhofmann
 * @version 
 */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.lang.*;
import java.io.*;

import scheduler.*;
import sysPr.*;

public class BrachaDeadlock extends java.applet.Applet implements Algorithm, Application, Runnable {

  static private final int NODES = 10;
  static private final int PROCESSESPERNODE = 1;
  static private final int SYSERR = 1;
  static private final int SPECIALPROCESSES = 0;
  
  static Scheduler scheduler;
  static Timer timer;
  
  private GridBagLayout gridBag;
  private GridBagConstraints c;
  private Button timeStepButton = new Button("next timestep(0)");
  private Button exitButton = new Button("exit");
  private Label label;
  private TextField[] textField = new TextField[NODES * PROCESSESPERNODE + SPECIALPROCESSES];
  private Button[] button = new Button[NODES * PROCESSESPERNODE + SPECIALPROCESSES];
  private Button[] applicationButton = new Button[NODES * PROCESSESPERNODE + SPECIALPROCESSES];
  private TextField[] syncState = new TextField[NODES * PROCESSESPERNODE + SPECIALPROCESSES];

  static public int simulationTime = 0;
  
  private Vector oldMessageQueue, messageQueue;
  private SimManager simulationManager;
  private CommunicationGraph communicationGraph;
  public HistoryFrame repFrame;
  public GraphFrame graphFrame;
  public AdjFrame adjFrame;
  
  SystemPrinter systemPrinter = new SystemPrinter();

  SystemThread[] systemThread = new SystemThread[NODES];
  
  private int rowNumber = 0;  // varaible for counting rows

  /**
  * @param args the command line arguments
  */
  public void init () {
    int i, j;
    gridBag = new GridBagLayout();
    c = new GridBagConstraints();
    setLayout(gridBag);

    // Create scheduler
    scheduler = new Scheduler(this, NODES * 2);

    messageQueue = new Vector();
    communicationGraph = new CommunicationGraph(getNumberOfNodes());

    // Create and name the processes
    for (i = 0; i < NODES; i++) {
      systemThread[i] = new SystemThread("S" + (i + 1), scheduler, i, communicationGraph);
    }
    
    // Add connections
    int[][] channels = {{1, 2}, {2, 3}, {3, 4}, {4, 5}, {5, 6}, {5, 7},
                        /*{6, 8},*/ {8, 3}, {8, 9}, {9,1}, {7, 10}};
    for (i = 0; i < channels.length; i++) {
      systemThread[channels[i][0] - 1].addConnection(systemThread[channels[i][1] - 1]);
      systemThread[channels[i][1] - 1].addConnection(systemThread[channels[i][0] - 1]);
      communicationGraph.addUnidirectionalConnection(channels[i][0] - 1, channels[i][1] - 1);
    }

    c.gridwidth = 8;
    c.fill = GridBagConstraints.HORIZONTAL;
    c.gridx = 0;
    c.gridy = rowNumber;
    c.weightx = 1.0;
    c.weighty = 1.0;
    gridBag.setConstraints(timeStepButton, c);
    add(timeStepButton);

    c.gridwidth = 8;
    c.fill = GridBagConstraints.HORIZONTAL;
    c.gridx = 0;
    c.gridy = rowNumber + 1;
    c.weightx = 1.0;
    c.weighty = 1.0;
    gridBag.setConstraints(exitButton, c);
    add(exitButton);

    validate();

    MyTimeStepListener myTimeStepListener = new MyTimeStepListener();
    timeStepButton.addActionListener(myTimeStepListener);

    MyExitListener myExitListener = new MyExitListener();
    exitButton.addActionListener(myExitListener);
    
    simulationManager = new SimManager(this);
    repFrame = new HistoryFrame(this);
    graphFrame = new GraphFrame(this);
    adjFrame = new AdjFrame(this, communicationGraph);
    adjFrame.mysim.changeNotify(this);

    // For scheduling purposes create intervall timer
    timer = new Timer(scheduler);
    timer.setName("timer");
           
    timer.start();
    scheduler.initialize();
  }

  public static void main(String args[]) {
    Frame f = new Frame("BrachaDeadlock");
    BrachaDeadlock system = new BrachaDeadlock();
    system.init();
    system.setSize(350, 400);
    system.setLocation(0,0);
    system.start();
    f.add("Center", system);
    f.setSize(350, 300);
    f.setLocation(0, 0);
    f.show();
  }

  public void updateState(int pid, String string, Color color, String buttonText) {
    textField[pid].setBackground(color);
    textField[pid].setText(string);
    button[pid].setLabel(buttonText);
  }
  
  public void updateButton(int pid, String label, boolean enabled, Color background) {
    button[pid].setEnabled(enabled);
    button[pid].setLabel(label);
    button[pid].setBackground(background);
  }

  public void updateSyncState(int pid, Color color) {
    syncState[pid].setBackground(color);
  }

  public void updateSyncState(int pid, String string, Color color) {
    syncState[pid].setText(string);
    syncState[pid].setBackground(color);
  }

  public SimManager getSimManager() {
    return simulationManager;
  }
  
  public int getSimulationTime() {
    return simulationTime;
  }
    
  public synchronized void enqueue(Message message) {
    messageQueue.addElement(message);
  }
    
  public int getNumberOfNodes() {
    return NODES;
  }
    
  public synchronized Vector getMessages() {
    return oldMessageQueue;
  }
    
  public int[] getDecisions() {
    return simulationManager.decisions;
  }
  
  public CommunicationGraph getCommunicationGraph() {
    return communicationGraph;
  }
  
  public void incrementClock(int t) {
    for (int i = 0; i < NODES; i++) {
    }
  }
    
  public void addRow(String string, int node, boolean withApplicationButton) {
    addLabel("Thread", rowNumber);
    addTextField("", rowNumber);
    addButton("resume", rowNumber);
    addSyncState("", rowNumber);
    rowNumber++;
  }
	
  public void addLabel(String name, int index) {
    label = new Label(name + index, Label.LEFT);
    c.gridx = 0;
    c.gridy = index;
    c.gridwidth = 1;
    c.weighty = 1.0;
    c.weightx = 1.0;
    gridBag.setConstraints(label, c);
    add(label);
  }
  
  public void addTextField(String string, int index) {
    textField[index] = new TextField(string, 8);
    textField[index].setEditable(true);
    c.gridx = 1;
    c.gridwidth = 1;
    c.weighty = 1.0;
    c.weightx = 1.0;
    gridBag.setConstraints(textField[index], c);
    add(textField[index]);
  }

  public void addButton(String string, int index) {
    button[index] = new Button(string);
    button[index].setEnabled(false);
    button[index].setBackground(Color.gray);
    button[index].addMouseListener(new MyMouseListener(index));
    c.gridx = 2;
    c.weighty = 1.0;
    c.weightx = 1.0;
    c.gridwidth = 1;
    gridBag.setConstraints(button[index], c);
    add(button[index]);
  }

  public void addSyncState(String string, int index) {
    syncState[index] = new TextField(string, 8);
    syncState[index].setEditable(true);
    c.gridx = 3;
    c.gridwidth = 1;
    c.weighty = 1.0;
    c.weightx = 1.0;
    gridBag.setConstraints(syncState[index], c);
    add(syncState[index]);
  }

  public void run() {
  }
  
  public Vector getOutgoingBuffers(int i) {
    Vector outgoingBuffers = new Vector();
    for (int j = 0; j < systemThread[i].outgoingConnections.size(); j++) {
      outgoingBuffers.add(((BasicConnection)(systemThread[i].outgoingConnections.elementAt(j))).buffer);
    }
    return outgoingBuffers;
  }
  
  public HistoryFrame getHistoryFrame() {
    return repFrame;
  }
  
  public GraphFrame getGraphFrame() {
    return graphFrame;
  }
  
  class MyMouseListener extends MouseAdapter {
    private int index;
    public MyMouseListener(int i) {
      index = i;
    }
    public synchronized void mousePressed(MouseEvent e) {
    }
    public synchronized void mouseReleased(MouseEvent e) {
      button[index].setLabel("");
      scheduler.resume(textField[index].getText());
    }
  }

  public void addEdge(Koord k) {
    systemThread[k.x].addConnection(systemThread[k.y]);
    systemThread[k.y].addConnection(systemThread[k.x]);
    communicationGraph.addUnidirectionalConnection(k.x, k.y);
    graphFrame.stateChanged();
  }

  public void deleteEdge(Koord k) {
    systemThread[k.x].deleteConnection(systemThread[k.y]);
    systemThread[k.y].deleteConnection(systemThread[k.x]);
    graphFrame.stateChanged();
  }

  class MyTimeStepListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      simulationTime++;
      timeStepButton.setLabel("next timestep(" + simulationTime + ")");
      oldMessageQueue = messageQueue;
      messageQueue = new Vector();
      simulationManager.nextTimestep(oldMessageQueue);
    }
  }

  class MyExitListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      SystemPrinter.close();
      System.exit(1);
    }
  }
  
}