import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;

import sysPr.*;

// Outputting diagram of received messages
public class History extends Canvas implements MouseMotionListener, MouseListener {
  int flag,flag1;
  public HistoryLayout lay;
  int horizontalSize = 50; // The default initial horizontal size of a round is set as 50.
  int verticalSize = 10;   // The default initial vertical size of a round is set as 10.
  Vector rounds;
  int numberOfRound;
  int numberOfNodes;
  Algorithm system;
  
  public History(int numberOfNodes,Algorithm sm) {
    flag1 = 0;
    flag = 0;
    this.numberOfNodes = numberOfNodes;
    numberOfRound = 0;
    // Vector with initial length 50 and increment 20
    rounds = new Vector(50, 20);
    // Create initial node symbols
    HistoryRound tmp = new HistoryRound(numberOfRound, numberOfNodes, sm.getSimManager().decisions);
    rounds.insertElementAt(tmp, numberOfRound);
    lay = new HistoryLayout(this);
    expandCanvas();
    this.addMouseListener(this);
    this.addMouseMotionListener(this);
    system = sm;
  }

  // Create the new HistoryRound, recalculate the size of the canvas and repaint
  public void simulateRound(GridMessage[] messages, int[] decisions) {
    numberOfRound++;
    rounds.insertElementAt(new HistoryRound(numberOfRound, messages, decisions), numberOfRound);
    lay.roundNext(messages);
    expandCanvas();
    repaint();
  }
  
  public void paint(Graphics g){
    ((HistoryRound) rounds.elementAt(0)).paint(g);
    // Draw vertical grid lines
    for(int i = 1; i <= numberOfRound; i++) {
      for(int j = 0 ; j < ((HistoryRound)rounds.elementAt(i)).mynodes.length; j++) {
	Color color = Color.black;
	if (j < ((HistoryRound)rounds.elementAt(i)).mynodes.length) {
	  switch (((HistoryRound)rounds.elementAt(i)).decision[j]) {
	    case 0: color = Color.black; break;
	    case 1: color = Color.green; break;
	    case 2: color = Color.red; break;
	    default: SystemPrinter.println("********** Unknown decision value");
	  }
	}
	g.setColor(color);
	g.drawLine(((HistoryRound) rounds.elementAt(i-1)).mynodes[j].getX(),
		   ((HistoryRound) rounds.elementAt(i-1)).mynodes[j].getY(),
		   ((HistoryRound) rounds.elementAt(i)).mynodes[j].getX(),
		   ((HistoryRound) rounds.elementAt(i)).mynodes[j].getY());
	  
      } 
      ((HistoryRound) rounds.elementAt(i)).paint(g);
    }
  }

  // Transform the Message given as parameter to a GridMessage
  public GridMessage transform(Message m) {
    int a = m.source.nodeNumber;
    int b = m.destination.nodeNumber;
    HistoryNode from =((HistoryRound)rounds.elementAt(numberOfRound)).mynodes[a];
    HistoryNode to = ((HistoryRound)rounds.elementAt(numberOfRound)).mynodes[b];
    GridMessage gm = new GridMessage(from, to);
    gm.sendingTime = m.creationTime;
    gm.setColor(m.getColor());
    return gm;
  }

  // Resize the Canvas with the current height and width
  public void expandCanvas() {
    setSize(getMaxX(),getMaxY());
  }
 
  public int getMaxX() {
    return ((HistoryRound) rounds.elementAt(0)).mynodes[numberOfNodes-1].mykoord.x + 100;
  }
  
  public int getMaxY() {
    return ((HistoryRound) rounds.elementAt(numberOfRound)).mynodes[numberOfNodes-1].mykoord.y + 100;
  }

  // Activate a flag when the mouse is clicked on an object, in this
  // case the last node of any round
  public void mousePressed(MouseEvent e) {
    if (((HistoryRound)rounds.elementAt(0)).mynodes[numberOfNodes-1].isIn(e.getPoint()))
      flag = 1;
    for (int i = 1 ; i <= numberOfRound ; i++) {
      if (((HistoryRound)rounds.elementAt(i)).mynodes[numberOfNodes-1].isIn(e.getPoint()))
	flag1 = i;
    }
  }
  
  // Deactivate the flag which was activated by mousePressed(e)
  public void mouseReleased(MouseEvent e) {
    flag = 0;
    flag1 = 0;
  }
  
  public void mouseEntered(MouseEvent e) {;}
  
  public void mouseClicked(MouseEvent e) {;}
  
  public void mouseExited(MouseEvent e) {;}

  // Control the different flags (only one can be active) and move the
  // considered part of the canvas
  public void mouseDragged(MouseEvent e){ 
    if(flag == 1) {
      lay.move_X(e.getX());
      repaint();
    }
    if(flag1 != 0) {
      lay.move_Y(e.getY(),flag1);
      repaint();
    }
  }
  public void mouseMoved(MouseEvent e){;}
  
  // Output graphics in Framemaker's MIF format
  public void printGraphic() {
    PrintWriter tmp;
    try {
      tmp = new PrintWriter(new FileOutputStream(new File("frameMIF.txt")));
      FileInputStream input = new FileInputStream(new File("d:\\prolog.txt"));
      int c = 0;
      while(c != -1) {
	c = input.read();
	if (c != -1) tmp.print((char)c);
      }
      System.out.println("file graphik copied");
      // Draw node on the upper border
      ((HistoryRound) rounds.elementAt(0)).printGraphic(tmp);
      // Print vertical grid lines
      for(int i = 1; i <= numberOfRound; i++) {
	for(int j = 0 ; j < ((HistoryRound) rounds.elementAt(i)).mynodes.length; j++) {
	  tmp.println(" <PolyLine ");
	  tmp.println("  <Unique 105694>");
	  tmp.println("  <Pen 0>");
	  tmp.println("  <PenWidth 1.0 pt>");
	  tmp.println("  <Separation 0>");
	  tmp.println("  <ObColor `Schwarz'>");
	  tmp.println("  <DashedPattern ");
	  tmp.println("   <DashedStyle Solid >");
	  tmp.println(" > # end of DashedPattern");
	  tmp.println("  <HeadCap Square >");
	  tmp.println("  <TailCap Square >");
	  tmp.println("  <Overprint No >");
	  tmp.println("  <NumPoints 2>");
	  tmp.println("  <Point  " + ((HistoryRound) rounds.elementAt(i-1)).mynodes[j].getX() + "pt "
			           + ((HistoryRound) rounds.elementAt(i-1)).mynodes[j].getY() + "pt>");
	  tmp.println("  <Point  " + ((HistoryRound) rounds.elementAt(i)).mynodes[j].getX() + "pt "
				   + ((HistoryRound) rounds.elementAt(i)).mynodes[j].getY() + "pt>");
	  tmp.println(" > # end of PolyLine");
	} 
        ((HistoryRound) rounds.elementAt(i)).printGraphic(tmp);
      }
      tmp.flush();
      input.close();
      input = new FileInputStream(new File("d:\\epilog.txt"));
      c = 0;
      while(c != -1) {
	c = input.read();
	if (c != -1) tmp.print((char)c);
      }
      tmp.close();
    } catch (IOException e) {
      System.out.println("" + e);
      System.out.println("PrintWriter tmp konnte nicht angelegt werden");
    }
  }
}






