import java.awt.*;
import java.util.Vector;
import java.util.Enumeration;

// ScrollRep integrates a Canvas (Repr) and adds the functionality
// of a Scrollbar
public class ScrollHistory extends ScrollPane  {
  
  History repr;
  Algorithm manager;
  
  // A SimManager is passed as a parameter to the constructor
  // to notify the events (for example: when a new Vector of Messages is ready
  public ScrollHistory(Algorithm sm) {
    super();
    manager = sm;
    repr = new History(sm.getNumberOfNodes(), sm);
    add(repr);
  }

  // The function simulateRound(GridMessage[] messages) just calls the
  // function simulateRound(GridMessage[] messages) of repr and sets
  // the position of the Canvas (repr) / Scrollbar 
  public void simulateRound(GridMessage[] messages, int[] decisions) {
    repr.simulateRound(messages, decisions);	
    setScrollPosition(getScrollPosition().x,
		      getVAdjustable().getMaximum() -
		      getViewportSize().height + 50);
    doLayout();
    repaint();
  }
  
  // Collect the vector of Messages given by the SimManager
  // and then call a (interface) Methode in Repr, which places each
  // received message into our GridMessage.
  public void notifyNextTimestep() {
    int i = 0;
    Vector messages_in = manager.getMessages();
    int[] decisions = manager.getDecisions();
    GridMessage [] myMessages = new GridMessage[messages_in.size()];
    for (Enumeration e = messages_in.elements(); e.hasMoreElements(); i++) {
      Message tmp = (Message)e.nextElement();
      myMessages[i] = repr.transform(tmp);
    }
    simulateRound(myMessages, decisions);
  }
  
  public void printGraphic() {
    repr.printGraphic();
  }
}







