import java.awt.*;

import scheduler.*;
import sysPr.*;

public class SystemThread extends SimProcess {
  int n = ((Algorithm)(scheduler.system)).getNumberOfNodes();
  int c;
  int needed = 0;
  int initiator = 0;
  int noGranted = 0;
  int notificationsSent = 0;
  int grantsSent = 0;
  boolean notified = false;
  boolean free = false;
  CommunicationGraph communicationGraph;
  
  public SystemThread(String name, Scheduler scheduler, int nodeNumber, CommunicationGraph communicationGraph) {
    super(name, scheduler, nodeNumber);
    ((Algorithm)(scheduler.system)).addRow(name, nodeNumber, false);
    scheduler.createThread(this);
    this.communicationGraph = communicationGraph;
  }
  
  public void runProcess() {
    int i;
    while (scheduler.system.getSimulationTime() < 1) scheduler.schedule();
    for (i = 0; i < n; i++) {
      if (communicationGraph.adjacenceMatrix[nodeNumber][i]) needed++;
    }
    c = outgoingConnections.size();
    if (nodeNumber == 0) {
      notifyServers();
      if (free) {
        SystemPrinter.println("\n++++++++++ System is not deadlocked ++++++++++");
      } else {
        SystemPrinter.println("---------- System is deadlocked ----------");
      }
    }

    while (true) {
      // SystemPrinter.println("Waiting for Message");
      Message message = receive();
      int sender = message.source.nodeNumber;
      
      if (acceptNotifyGrant(message)) {
      } else {
	SystemPrinter.println("Unknown message type; suspending myself");
	scheduler.suspend();
      }
    }
  }
    
  boolean acceptNotifyGrant(Message message) {
    if (message.type.equals("NOTIFY")) {
      if (!notified)
        notifyServers();
      for (int i = 0; i < c; i++) {
        Connection conn = null;
        if ((conn = ((Connection)(outgoingConnections.elementAt(i)))).destination == message.source) {
          conn.send("DONE");
          break;
        }
      }
      return true;
    } else if (message.type.equals("GRANT")) {
      noGranted++;
      if (!free && noGranted >= needed) grant();
      for (int i = 0; i < c; i++) {
        Connection conn = null;
        if ((conn = ((Connection)(outgoingConnections.elementAt(i)))).destination == message.source) {
          conn.send("ACK");
          break;
        }
      }
     // send(u, ACK);
      return true;
    } else return false;
  }
  
  boolean await(String string) {
    Message message = receive();
    if (acceptNotifyGrant(message)) return false;
    else if (message.type.equals(string)) return true;
    else return false;
  }
  
  void notifyServers() {
    notified = true;
    for (int i = 0; i < c; i++) {
      Connection connection = ((Connection)(outgoingConnections.elementAt(i)));
      if (communicationGraph.adjacenceMatrix[nodeNumber][connection.destination.nodeNumber]) {
        connection.send("NOTIFY");
        notificationsSent++;
      }
    }
    if (c == 1) {
      // I'm leave
      grant();
    }
    // for (all w  OUT) await(w, DONE);
    int i = 0;
    while (i < notificationsSent) {
      if (await("DONE")) i++;
    }
  }

  void grant() {
    free = true;
    // for (all w  IN) send(w, GRANT);
    for (int i = 0; i < c; i++) {
      Connection connection = ((Connection)(outgoingConnections.elementAt(i)));
      if (communicationGraph.adjacenceMatrix[connection.destination.nodeNumber][nodeNumber]) {
        connection.send("GRANT");
        grantsSent++;
      }
    }
    // for (all w  IN) await(w, ACK);
    int i = 0;
    while (i < grantsSent) {
      if (await("ACK")) i++;
    }
  }
}
